// Copyright (c) 2009 Divmod.
// See LICENSE for details.

// import Nevow.Athena

/**
 * A dummy module used in some dependency tests.
 */

Nevow.Athena.Widget.subclass('Mantissa.Test.Dummy', DummyWidget);
